import { Auth } from '@genesislcap/foundation-comms';
import { AvailableOption } from '@genesislcap/foundation-ui';
import { ColDef } from '@ag-grid-community/core';
import type { ColGroupDef } from '@ag-grid-community/core';
import { Connect } from '@genesislcap/foundation-comms';
import { Constructable } from '@microsoft/fast-element';
import { Container } from '@microsoft/fast-foundation';
import { Controller } from '@microsoft/fast-element';
import { DatasourceOptions } from '@genesislcap/foundation-comms';
import { DOMContainer } from '@genesislcap/foundation-utils';
import { FASTElement } from '@microsoft/fast-element';
import { FoundationAnalytics } from '@genesislcap/foundation-comms';
import { FoundationLayoutContainer } from '@genesislcap/foundation-utils';
import { GridOptions } from '@ag-grid-community/core';
import type { GridProCaseType } from '@genesislcap/grid-pro';
import { GridProRendererTypes } from '@genesislcap/foundation-zero-grid-pro';
import { LayoutCacheContainer } from '@genesislcap/foundation-utils';
import { Logger } from '@genesislcap/foundation-logger';
import type { Modal } from '@genesislcap/foundation-ui';
import { PartialFASTElementDefinition } from '@microsoft/fast-element';
import { RendererEntry } from '@genesislcap/foundation-forms';
import { RouterConfiguration } from '@microsoft/fast-router';
import { SelectedOption } from '@genesislcap/foundation-ui';
import { Session } from '@genesislcap/foundation-comms';
import { UiSchema } from '@genesislcap/foundation-forms';
import { ViewTemplate } from '@microsoft/fast-element';

declare type AdditionalEntry = string | UiSchema | {
    type: 'Control';
    scope: string;
    label?: string;
    options?: {
        readonly?: boolean;
        hidden?: boolean;
        allOptionsResourceName?: string;
        valueField?: string;
        labelField?: string;
    };
};

/**
 * @public
 */
export declare const buttonCellRenderer: (title: any, clickHandler: any, contentTemplate: any) => ColDef;

/**
 * The attribute which is set to configure the resource that the entity manager is working with
 *
 * @public
 *
 * @example
 * ```javascript
 * type DatasourceConfiguration = {
 *   criteria?: string;
 *   fields?: string;
 *   isSnapshot?: boolean;
 *   maxRows?: number;
 *   maxView?: number;
 *   movingView?: boolean;
 *   pollingInterval?: number;
 *   orderBy?: string;
 *   reverse?: boolean;
 * }
 * ```
 */
export declare type DatasourceConfiguration = Omit<DatasourceOptions, 'resourceName'>;

/**
 * @internal
 */
export declare const DynamicTemplate: ViewTemplate<MainApplication>;

/**
 * Main class which defines the entity management functionality
 *
 * @remarks
 *
 * Connects to a backend resource and wraps up a grid which is populated
 * with entities from that resource. The different interactions that the user
 * can perform with the entities can be configured, examples being able to
 * update and delete entities.
 *
 * @public
 *
 * @example
 * Example of using the entity management system to handle counterparties
 * ```
 * <entity-management
 *   resourceName="ALL_COUNTERPARTYS"
 *   title="Counterparty Management"
 *   updateEvent="EVENT_COUNTERPARTY_MODIFY"
 *   deleteEvent="EVENT_COUNTERPARTY_DELETE"
 *   createEvent="EVENT_COUNTERPARTY_INSERT"
 * ></entity-management>
 * ```
 * Where:<br />
 * - the title of the grid is `Counterparty Management`<br />
 * - the name of the resource in the database to manage is `ALL_COUNTERPARTYS`<br />
 * - the name of the event handler for update events is `EVENT_COUNTERPARTY_MODIFY`<br />
 * - the name of the event handler for create events is `EVENT_COUNTERPARTY_INSERT`<br />
 * - the name of the event handler for delete events is `EVENT_COUNTERPARTY_DELETE`<br />
 *
 */
export declare class EntityManagement extends EntityManagement_base {
    /**
     * DI connect object which is used to interact with the backend.
     *
     * @remarks
     *
     * Requires {@link @genesislcap/foundation-comms#Connect | the connect class} to be registered with the
     * DI container
     */
    protected connect: Connect;
    /**
     * Name of the design system prefix that will be used in renderers.
     * @public
     */
    prefix: string;
    /**
     * Name of the backend resource which contain the entities to manage
     * @public
     */
    resourceName: string;
    readEvent: string;
    readEventFn: (...args: any[]) => {};
    readonly: boolean;
    /**
     * Name of the event handler on the Genesis server which handles updating the entity
     * @public
     */
    updateEvent: string;
    /**
     * Name of the event handler on the Genesis server which handles deleting the entity
     * @public
     */
    deleteEvent: string;
    /**
     * Name of the event handler on the Genesis server which handles creating an entity
     * @public
     */
    createEvent: string;
    /**
     * Title of the grid
     * @public
     */
    title: string;
    /**
     * Label for the entity which has usages such as being shown in the title of the modal wen editing the entity
     * @public
     */
    entityLabel: string;
    /**
     * Whether to use the `applyTransactionAsync` function for *add* transactions
     * @remarks Defaults to false
     * @public
     */
    asyncAdd: boolean;
    /**
     * Whether to use the `applyTransactionAsync` function for *remove* transactions
     * @remarks Defaults to false
     * @public
     */
    asyncRemove: boolean;
    /**
     * Whether to use the `applyTransactionAsync` function for *update* transactions
     * @remarks Defaults to true
     * @public
     */
    asyncUpdate: boolean;
    /**
     * This attribute allows you to change row-selection to 'single' or 'multiple' which will allow you to select single or multiple rows.
     * @remarks Defaults to single
     * @public
     */
    rowSelection: string;
    /**
     * If false, will disable cell flashing for all cells by default, unless otherwise defined in custom colDef
     * @remarks Defaults to true
     * @public
     */
    enableCellFlashing: boolean;
    /**
     * If true, will enable row flashing for all rows for `add` transactions
     * @public
     */
    enableRowFlashing: boolean;
    /**
     * This attribute controls whether and how the entity manager stores the state of the columns when the user edits them. Omit this attribute to disable the functionality, set it to a unique value to enable it.
     *
     * @remarks
     *
     * Setting this value will set the entity manager to persist the column states through page refreshes etc. An example of what is stored is when the user resizes or reorders columns.
     * This value must be unique for each table in your app otherwise the persisted data will be corrupted.
     * There is an option on the grid for the user to reset the table to the default layout if they wish.
     * If you omit this attribute then nothing is persisted
     * @public
     */
    persistColumnStateKey: string;
    /**
     * GridOptions to be passed down from application
     * @public
     */
    gridOptions: GridOptions;
    /**
     * Array which holds the column definitions.
     * @public
     */
    columns: ColDef[];
    /**
     * The configuration which is used when interacting with the resource on the backend
     *
     * @remarks
     *
     * Handles the configuration such as how many rows of entities to fetch at a time
     * @public
     */
    datasourceConfig: DatasourceConfiguration;
    formUiSchema: UiSchema;
    createFormUiSchema: UiSchema;
    updateFormUiSchema: UiSchema;
    /**
     * The name of the resource in the backend to interact with when dispatching actions from the update/create modal
     * @internal
     */
    formResourceName: string;
    /**
     * Array with renderers used by foundation-forms
     * @public
     */
    formRenderers: RendererEntry[];
    /**
     * Reference to the currently selected entity from the grid.
     *
     * @remarks
     *
     * When the user attempts to initiate functionality such as deleting or editing an entity, the entity which will be deleted/edited is the one which is referenced by this variable
     * @public
     */
    selectedEntity: Record<string, unknown>;
    /**
     * Disables the form while enabled to stop the user dispatching a large number of duplicate events
     * @public
     */
    editedEntity: Record<string, unknown>;
    /**
     * The default values to populate the form with when the user is adding an entity
     * @public
     */
    defaultEntityValues: Record<string, unknown>;
    submitting: boolean;
    /**
     * String which contains the text of the popup modal when the user is adding or editing an entity
     */
    editDialogTitle: string;
    editModalVisible: boolean;
    editModalVisibleChanged(): void;
    editEntityModal: Modal;
    sizeColumnsToFit: boolean;
    enableFilterBar: boolean;
    hideEdit: boolean;
    hideDelete: boolean;
    /**
     * This attribute controls whether to enable the search-bar.
     * @public
     */
    enableSearchBar: boolean;
    /**
     * The case type to use for the header names. If not set, the default CONSTANT_CASE will be used.
     * @remarks Can be one of the following: camelCase, capitalCase, dotCase, headerCase, noCase, paramCase, pascalCase, pathCase, sentenceCase, snakeCase.
     * @public
     */
    headerCaseType: GridProCaseType;
    searchBarConfig: AvailableOption[];
    /**
     * Determines where the modal dialog will appear on screen
     * @public
     */
    modalPosition: 'centre' | 'left' | 'right';
    /**
     * Set up the web component
     * @internal
     */
    connectedCallback(): Promise<void>;
    disconnectedCallback(): Promise<void>;
    /**
     * Gets the default searchBarConfig if the user has not created their own.
     * @internal
     */
    private getDefaultSearchBarConfig;
    /**
     * Override the deepClone method to ensure that observable attributes are cloned
     * @public
     */
    deepClone(): Node;
    /**
     * Event handler for when the user submits the action for the currently open form, either editing or adding the entity
     *
     * @param e - CustomEvent which contains the payload for the entity to submit to the backend
     *
     * Emits an event upon error
     * @public
     */
    submitEntityChanges(e: CustomEvent): Promise<void>;
    /**
     * Updates the reference to the current entity stored in the class. Added as an event listener on the class when receiving the `rowSelected` event.
     *
     * @param e - CustomEvent which contains data about the selected entity. If the row is not selected then the reference will be set to an empty object.
     * @internal
     */
    private selectEntity;
    /**
     * Show the entity create form
     * Sets the initial value on the form to be defaultEntityValues if present.
     * @internal
     */
    createEntity(): void;
    /**
     * Opens the model to edit an entity. Added as an event listener on the class when receiving the `edit-entity` event.
     *
     * @param e - CustomEvent where `e.detail` is the entity to edit.
     * @internal
     */
    private editEntity;
    private readEntity;
    closeModal(): void;
    get headerTempalate(): ViewTemplate<EntityManagement, any>;
    searchChanged(event: CustomEvent<Array<SelectedOption>>): void;
    searchBarReqServer(options: SelectedOption<string>[]): void;
    searchBarDataserver(options: SelectedOption<string>[]): void;
    requestChanged(e: CustomEvent<string>): void;
    criteriaChanged(e: CustomEvent<string>): void;
    submitFailureNotification(e: CustomEvent): void;
    /**
     * Handler for deleting the entity. Added as an event listener on the class when receiving the `delete-entity` event.
     *
     * @param e - CustomEvent where `e.detail` contains data to update the selectedEntity reference with the entity to delete.
     * @internal
     */
    deleteEntity(e: CustomEvent): Promise<void>;
    confirmDelete(): Promise<void>;
    private showDeleteConfirmation;
    /**
     * Show notifications if the executed event returned an error.
     * @internal
     */
    errorNotify(request: any): void;
}

declare const EntityManagement_base: (new (...args: any[]) => {
    "__#1@#_container": FoundationLayoutContainer | LayoutCacheContainer | DOMContainer;
    "__#1@#_latestTokenCode": string;
    "__#1@#_hasFirstLoaded": boolean;
    "__#1@#_cleanupTimeout": NodeJS.Timeout;
    "__#1@#_shouldForceLifecycle": boolean;
    cloneNode(deep?: boolean): Node;
    deepClone(): Node;
    readonly shouldRunDisconnect: boolean;
    readonly shouldRunConnect: boolean;
    "__#1@#_blockLifecycleDueToTokenChange"(lifecycleType: "connect" | "disconnect" | "reconnect"): boolean;
    "__#1@#_tryFindContainingLayout"(e: Element): FoundationLayoutContainer | LayoutCacheContainer | DOMContainer;
    connectedCallback(): void;
    readonly $fastController: Controller;
    $emit(type: string, detail?: any, options?: Omit<CustomEventInit<any>, "detail">): boolean | void;
    disconnectedCallback(): void;
    attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
    accessKey: string;
    readonly accessKeyLabel: string;
    autocapitalize: string;
    dir: string;
    draggable: boolean;
    hidden: boolean;
    inert: boolean;
    innerText: string;
    lang: string;
    readonly offsetHeight: number;
    readonly offsetLeft: number;
    readonly offsetParent: Element;
    readonly offsetTop: number;
    readonly offsetWidth: number;
    outerText: string;
    spellcheck: boolean;
    title: string;
    translate: boolean;
    attachInternals(): ElementInternals;
    click(): void;
    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
    removeEventListener<K_1 extends keyof HTMLElementEventMap>(type: K_1, listener: (this: HTMLElement, ev: HTMLElementEventMap[K_1]) => any, options?: boolean | EventListenerOptions): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
    readonly attributes: NamedNodeMap;
    readonly classList: DOMTokenList;
    className: string;
    readonly clientHeight: number;
    readonly clientLeft: number;
    readonly clientTop: number;
    readonly clientWidth: number; /**
    * Array which holds the column definitions.
    * @public
    */
    id: string;
    readonly localName: string;
    readonly namespaceURI: string;
    onfullscreenchange: (this: Element, ev: Event) => any;
    onfullscreenerror: (this: Element, ev: Event) => any;
    outerHTML: string;
    readonly ownerDocument: Document;
    readonly part: DOMTokenList;
    readonly prefix: string;
    readonly scrollHeight: number;
    scrollLeft: number;
    scrollTop: number;
    readonly scrollWidth: number;
    readonly shadowRoot: ShadowRoot;
    slot: string;
    readonly tagName: string;
    attachShadow(init: ShadowRootInit): ShadowRoot;
    closest<K_2 extends keyof HTMLElementTagNameMap>(selector: K_2): HTMLElementTagNameMap[K_2];
    closest<K_3 extends keyof SVGElementTagNameMap>(selector: K_3): SVGElementTagNameMap[K_3];
    closest<E extends Element = Element>(selectors: string): E;
    getAttribute(qualifiedName: string): string;
    getAttributeNS(namespace: string, localName: string): string;
    getAttributeNames(): string[];
    getAttributeNode(qualifiedName: string): Attr;
    getAttributeNodeNS(namespace: string, localName: string): Attr;
    getBoundingClientRect(): DOMRect; /**
    * Disables the form while enabled to stop the user dispatching a large number of duplicate events
    * @public
    */
    getClientRects(): DOMRectList;
    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;
    getElementsByTagName<K_4 extends keyof HTMLElementTagNameMap>(qualifiedName: K_4): HTMLCollectionOf<HTMLElementTagNameMap[K_4]>;
    getElementsByTagName<K_5 extends keyof SVGElementTagNameMap>(qualifiedName: K_5): HTMLCollectionOf<SVGElementTagNameMap[K_5]>;
    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;
    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;
    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;
    getElementsByTagNameNS(namespace: string, localName: string): HTMLCollectionOf<Element>;
    hasAttribute(qualifiedName: string): boolean;
    hasAttributeNS(namespace: string, localName: string): boolean;
    hasAttributes(): boolean;
    hasPointerCapture(pointerId: number): boolean;
    insertAdjacentElement(where: InsertPosition, element: Element): Element;
    insertAdjacentHTML(position: InsertPosition, text: string): void;
    insertAdjacentText(where: InsertPosition, data: string): void;
    matches(selectors: string): boolean;
    releasePointerCapture(pointerId: number): void;
    removeAttribute(qualifiedName: string): void;
    removeAttributeNS(namespace: string, localName: string): void;
    removeAttributeNode(attr: Attr): Attr;
    requestFullscreen(options?: FullscreenOptions): Promise<void>;
    requestPointerLock(): void;
    scroll(options?: ScrollToOptions): void;
    scroll(x: number, y: number): void;
    scrollBy(options?: ScrollToOptions): void;
    scrollBy(x: number, y: number): void;
    scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;
    scrollTo(options?: ScrollToOptions): void;
    scrollTo(x: number, y: number): void;
    setAttribute(qualifiedName: string, value: string): void;
    setAttributeNS(namespace: string, qualifiedName: string, value: string): void;
    setAttributeNode(attr: Attr): Attr;
    setAttributeNodeNS(attr: Attr): Attr;
    setPointerCapture(pointerId: number): void;
    toggleAttribute(qualifiedName: string, force?: boolean): boolean;
    webkitMatchesSelector(selectors: string): boolean;
    readonly baseURI: string;
    readonly childNodes: NodeListOf<ChildNode>;
    readonly firstChild: ChildNode;
    readonly isConnected: boolean;
    readonly lastChild: ChildNode;
    readonly nextSibling: ChildNode;
    readonly nodeName: string;
    readonly nodeType: number;
    nodeValue: string;
    readonly parentElement: HTMLElement;
    readonly parentNode: ParentNode;
    readonly previousSibling: ChildNode;
    textContent: string;
    appendChild<T_1 extends Node>(node: T_1): T_1;
    compareDocumentPosition(other: Node): number;
    contains(other: Node): boolean;
    getRootNode(options?: GetRootNodeOptions): Node;
    hasChildNodes(): boolean;
    insertBefore<T_2 extends Node>(node: T_2, child: Node): T_2;
    isDefaultNamespace(namespace: string): boolean;
    isEqualNode(otherNode: Node): boolean;
    isSameNode(otherNode: Node): boolean;
    lookupNamespaceURI(prefix: string): string;
    lookupPrefix(namespace: string): string;
    normalize(): void;
    removeChild<T_3 extends Node>(child: T_3): T_3;
    replaceChild<T_4 extends Node>(node: Node, child: T_4): T_4;
    readonly ATTRIBUTE_NODE: number;
    readonly CDATA_SECTION_NODE: number;
    readonly COMMENT_NODE: number;
    readonly DOCUMENT_FRAGMENT_NODE: number;
    readonly DOCUMENT_NODE: number;
    readonly DOCUMENT_POSITION_CONTAINED_BY: number;
    readonly DOCUMENT_POSITION_CONTAINS: number;
    readonly DOCUMENT_POSITION_DISCONNECTED: number;
    readonly DOCUMENT_POSITION_FOLLOWING: number;
    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;
    readonly DOCUMENT_POSITION_PRECEDING: number;
    readonly DOCUMENT_TYPE_NODE: number;
    readonly ELEMENT_NODE: number;
    readonly ENTITY_NODE: number;
    readonly ENTITY_REFERENCE_NODE: number;
    readonly NOTATION_NODE: number;
    readonly PROCESSING_INSTRUCTION_NODE: number;
    readonly TEXT_NODE: number;
    dispatchEvent(event: Event): boolean;
    ariaAtomic: string;
    ariaAutoComplete: string;
    ariaBusy: string;
    ariaChecked: string;
    ariaColCount: string;
    ariaColIndex: string;
    ariaColIndexText: string;
    ariaColSpan: string;
    ariaCurrent: string;
    ariaDisabled: string;
    ariaExpanded: string;
    ariaHasPopup: string;
    ariaHidden: string;
    ariaInvalid: string;
    ariaKeyShortcuts: string;
    ariaLabel: string;
    ariaLevel: string;
    ariaLive: string;
    ariaModal: string;
    ariaMultiLine: string;
    ariaMultiSelectable: string;
    ariaOrientation: string;
    ariaPlaceholder: string;
    ariaPosInSet: string;
    ariaPressed: string;
    ariaReadOnly: string;
    ariaRequired: string;
    ariaRoleDescription: string;
    ariaRowCount: string;
    ariaRowIndex: string;
    ariaRowIndexText: string;
    ariaRowSpan: string;
    ariaSelected: string;
    ariaSetSize: string;
    ariaSort: string;
    ariaValueMax: string;
    ariaValueMin: string;
    ariaValueNow: string;
    ariaValueText: string;
    role: string;
    animate(keyframes: PropertyIndexedKeyframes | Keyframe[], options?: number | KeyframeAnimationOptions): Animation;
    getAnimations(options?: GetAnimationsOptions): Animation[];
    after(...nodes: (string | Node)[]): void;
    before(...nodes: (string | Node)[]): void;
    remove(): void;
    replaceWith(...nodes: (string | Node)[]): void; /**
    * Show the entity create form
    * Sets the initial value on the form to be defaultEntityValues if present.
    * @internal
    */
    innerHTML: string;
    readonly nextElementSibling: Element;
    readonly previousElementSibling: Element;
    readonly childElementCount: number;
    readonly children: HTMLCollection;
    readonly firstElementChild: Element;
    readonly lastElementChild: Element;
    append(...nodes: (string | Node)[]): void;
    prepend(...nodes: (string | Node)[]): void;
    querySelector<K_6 extends keyof HTMLElementTagNameMap>(selectors: K_6): HTMLElementTagNameMap[K_6];
    querySelector<K_7 extends keyof SVGElementTagNameMap>(selectors: K_7): SVGElementTagNameMap[K_7];
    querySelector<E_1 extends Element = Element>(selectors: string): E_1;
    querySelectorAll<K_8 extends keyof HTMLElementTagNameMap>(selectors: K_8): NodeListOf<HTMLElementTagNameMap[K_8]>;
    querySelectorAll<K_9 extends keyof SVGElementTagNameMap>(selectors: K_9): NodeListOf<SVGElementTagNameMap[K_9]>;
    querySelectorAll<E_2 extends Element = Element>(selectors: string): NodeListOf<E_2>;
    replaceChildren(...nodes: (string | Node)[]): void;
    readonly assignedSlot: HTMLSlotElement;
    oncopy: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    oncut: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    onpaste: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    readonly style: CSSStyleDeclaration;
    contentEditable: string;
    enterKeyHint: string;
    inputMode: string;
    readonly isContentEditable: boolean;
    onabort: (this: GlobalEventHandlers, ev: UIEvent) => any;
    onanimationcancel: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationend: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationiteration: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationstart: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onauxclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onbeforeinput: (this: GlobalEventHandlers, ev: InputEvent) => any;
    onblur: (this: GlobalEventHandlers, ev: FocusEvent) => any;
    oncancel: (this: GlobalEventHandlers, ev: Event) => any;
    oncanplay: (this: GlobalEventHandlers, ev: Event) => any;
    oncanplaythrough: (this: GlobalEventHandlers, ev: Event) => any;
    onchange: (this: GlobalEventHandlers, ev: Event) => any;
    onclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onclose: (this: GlobalEventHandlers, ev: Event) => any;
    oncontextmenu: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    oncuechange: (this: GlobalEventHandlers, ev: Event) => any;
    ondblclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    ondrag: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragend: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragenter: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragleave: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragover: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragstart: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondrop: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondurationchange: (this: GlobalEventHandlers, ev: Event) => any;
    onemptied: (this: GlobalEventHandlers, ev: Event) => any;
    onended: (this: GlobalEventHandlers, ev: Event) => any;
    onerror: OnErrorEventHandlerNonNull;
    onfocus: (this: GlobalEventHandlers, ev: FocusEvent) => any;
    onformdata: (this: GlobalEventHandlers, ev: FormDataEvent) => any;
    ongotpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    oninput: (this: GlobalEventHandlers, ev: Event) => any;
    oninvalid: (this: GlobalEventHandlers, ev: Event) => any;
    onkeydown: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onkeypress: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onkeyup: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onload: (this: GlobalEventHandlers, ev: Event) => any;
    onloadeddata: (this: GlobalEventHandlers, ev: Event) => any;
    onloadedmetadata: (this: GlobalEventHandlers, ev: Event) => any;
    onloadstart: (this: GlobalEventHandlers, ev: Event) => any;
    onlostpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onmousedown: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseenter: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseleave: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmousemove: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseout: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseover: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseup: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onpause: (this: GlobalEventHandlers, ev: Event) => any;
    onplay: (this: GlobalEventHandlers, ev: Event) => any;
    onplaying: (this: GlobalEventHandlers, ev: Event) => any;
    onpointercancel: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerdown: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerenter: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerleave: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointermove: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerout: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerover: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerup: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onprogress: (this: GlobalEventHandlers, ev: ProgressEvent<EventTarget>) => any;
    onratechange: (this: GlobalEventHandlers, ev: Event) => any;
    onreset: (this: GlobalEventHandlers, ev: Event) => any;
    onresize: (this: GlobalEventHandlers, ev: UIEvent) => any;
    onscroll: (this: GlobalEventHandlers, ev: Event) => any;
    onsecuritypolicyviolation: (this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any;
    onseeked: (this: GlobalEventHandlers, ev: Event) => any;
    onseeking: (this: GlobalEventHandlers, ev: Event) => any;
    onselect: (this: GlobalEventHandlers, ev: Event) => any;
    onselectionchange: (this: GlobalEventHandlers, ev: Event) => any;
    onselectstart: (this: GlobalEventHandlers, ev: Event) => any;
    onslotchange: (this: GlobalEventHandlers, ev: Event) => any;
    onstalled: (this: GlobalEventHandlers, ev: Event) => any;
    onsubmit: (this: GlobalEventHandlers, ev: SubmitEvent) => any;
    onsuspend: (this: GlobalEventHandlers, ev: Event) => any;
    ontimeupdate: (this: GlobalEventHandlers, ev: Event) => any;
    ontoggle: (this: GlobalEventHandlers, ev: Event) => any;
    ontouchcancel?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchend?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchmove?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchstart?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontransitioncancel: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionend: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionrun: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionstart: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    onvolumechange: (this: GlobalEventHandlers, ev: Event) => any;
    onwaiting: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationend: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationiteration: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationstart: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkittransitionend: (this: GlobalEventHandlers, ev: Event) => any;
    onwheel: (this: GlobalEventHandlers, ev: WheelEvent) => any;
    autofocus: boolean;
    readonly dataset: DOMStringMap;
    nonce?: string;
    tabIndex: number;
    blur(): void;
    focus(options?: FocusOptions): void;
}) & (new () => HTMLElement & FASTElement) & {
    from<TBase extends {
        new (): HTMLElement;
        prototype: HTMLElement;
    }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
    define<TType extends Function>(type: TType, nameOrDef?: string | PartialFASTElementDefinition): TType;
};

/**
 * Format the error as a string with a different format depending on whether
 * `err` has a `FIELD` property or not
 * @public
 */
export declare const getErrorFormat: (err: any) => string;

/**
 * @internal
 */
export declare const LoadingTemplate: ViewTemplate<MainApplication>;

/**
 * @public
 */
export declare const logger: Logger;

/**
 * @public
 */
export declare class MainApplication extends FASTElement {
    config: MainRouterConfig;
    connect: Connect;
    session: Session;
    container: Container;
    provider: HTMLElement;
    ready: boolean;
    connectedCallback(): void;
    onLuminanceToggle(): void;
    loadRemotes(): Promise<void>;
    selectTemplate(): ViewTemplate<MainApplication, any>;
    private registerDIDependencies;
}

/**
 * Base implementation of a routing used with the Login class
 *
 * @remarks
 *
 * Could be used when using the Login class as a standalone authentication method
 * with your app
 *
 * @internal
 */
declare class MainRouterConfig extends RouterConfiguration<RouterSettings> {
    private auth;
    private container;
    private analytics;
    private session;
    constructor(auth: Auth, container: Container, analytics: FoundationAnalytics, session: Session);
    configure(): void;
    construct<T>(Type: Constructable<T>): T;
}

/**
 * @internal
 */
export declare const MainTemplate: ViewTemplate<MainApplication>;

/**
 * Main class which defined the profile management functionality
 *
 * @deprecated - Use '\@genesislcap/pbc-auth' instead.
 * 'https://www.npmjs.com/package/\@genesislcap/pbc-auth'
 * @remarks
 *
 * Under the hood the profile-management is the entity-management micro-frontend configured for use
 * with the `ALL_PROFILES` resource.
 *
 * @public
 *
 * @example
 *
 * You can use this class by forward declaring it in the template file where it is used, and then using
 * it in a html template. It is recommended to wrap it in an error boundary.
 * ```javascript
 * <profile-management></profile-management>
 * ```
 *
 */
export declare class Profiles extends EntityManagement {
    /**
     * Auth provider which is used to confirm authorisation of the user for their user management permissions
     *
     * @remarks
     *
     * Requires {@link @genesislcap/foundation-comms#Auth | the auth class} to be registered with the
     * DI container
     * @internal
     */
    private auth;
    /**
     * Constructor will set the {@link EntityManagement.createEvent} {@link EntityManagement.deleteEvent} {@link EntityManagement.updateEvent} to match the permissions set from the server
     */
    constructor();
    readProfileData(profile: any): Promise<{
        RIGHT_CODES: any[];
        USER_NAMES: any[];
        NAME: any;
    }>;
    confirmDelete(): Promise<void>;
}

declare type RouterSettings = {
    public?: boolean;
    autoAuth?: boolean;
};

/**
 * Main class which defines the user management functionality
 *
 * @remarks
 *
 * Under the hood the user-management is the entity-management micro-frontend configured for use
 * with the `ALL_USERS` resource.
 * @deprecated - Use '\@genesislcap/pbc-auth' instead.
 * 'https://www.npmjs.com/package/\@genesislcap/pbc-auth'
 * @public
 *
 * @example
 *
 * With no customisation required the user management micro-frontend can simply be dropped into the
 * template as shown. However, it is advised to wrap it in an error boundary component.
 * ```javascript
 * <user-management></user-management>
 * ```
 * With this default example above the columns will be the {@link UsersColumnConfig} as well as the
 * entity and status columns. The entity and status
 * columns are always added regardless of the other columns which are configured.
 *
 * You can add in additional customisation options. For example you can define a custom column and
 * then mix-in with the standard columns
 * ```javascript
 * import { Users, UsersColumnConfig, } from '@genesislcap/foundation-entity-management';
 *
 * const userColumns = [
 *   {
 *     field: 'ACCESS_TYPE',
 *     headerName: 'Access Type',
 *   },
 * ];
 *
 * // Definition inside of the html template
 * <user-management :columns=${() => [...UsersColumnConfig, ...userColumns]}></user-management>
 * ```
 */
export declare class Users extends Users_base {
    /**
     * DI connect object which is used to interact with the backend.
     *
     * @remarks
     *
     * Requires {@link @genesislcap/foundation-comms#Connect | the connect class} to be registered with the
     * DI container
     * @internal
     */
    private connect;
    /**
     * Auth provider which is used to confirm authorisation of the user for their user management permissions
     *
     * @remarks
     *
     * Requires {@link @genesislcap/foundation-comms#Auth | the auth class} to be registered with the
     * DI container
     * @internal
     */
    private auth;
    /**
     * The configuration which is used when interacting with the resource on the backend
     *
     * @remarks
     *
     * Handles the configuration such as how many rows of entities to fetch at a time
     * @internal
     */
    datasourceConfig: DatasourceConfiguration;
    /**
     * Name of the design system prefix that will be used for components.
     * @public
     */
    prefix: string;
    /**
     * Configuring the resourceName from the entity-management, set to manage the users resource specifically
     * @internal
     */
    resourceName: string;
    /**
     * This attribute controls whether and how the user manager stores the state of the columns when the user edits them. Defaulted to `entity_users_management`.
     *
     * @remarks
     *
     * Setting this value will set the entity manager to persist the column states through page refreshes etc. An example of what is stored is when the user resizes or reorders columns.
     * This value must be unique for each table in your app otherwise the persisted data will be corrupted.
     * There is an option on the grid for the user to reset the table to the default layout if they wish.
     * @public
     */
    persistColumnStateKey: string;
    /**
     * This attribute controls whether to enable the search-bar.
     * @public
     */
    enableSearchBar: boolean;
    /**
     * GridOptions to be passed down from application
     * @public
     */
    gridOptions: GridOptions;
    /**
     * Returns a configuration for an optional status column
     *
     * @returns Column configuration for an optional status column
     * @internal
     */
    statusColumn(): {
        headerName: string;
        field: string;
        minWidth: number;
        width: number;
        suppressSizeToFit: boolean;
        cellRendererSelector: any;
        enableCellChangeFlash: boolean;
    };
    /**
     * Returns the config for an entity column if the entityID attribute is defined
     *
     * @returns Config for an entity column if the entityID is defined, else an empty array
     * @internal
     */
    entityColumn(): {
        headerName: string;
        field: string;
        enableCellChangeFlash: boolean;
    }[];
    /**
     * Column definition, default to the UsersColumnConfig
     */
    columns: (ColDef<any> | ColGroupDef<any>)[];
    additionalCreateFields: AdditionalEntry[];
    additionalUpdateFields: AdditionalEntry[];
    allAccessType: boolean;
    private entityIdValue;
    /**
     * String which contains event if we have permission to insert user of empty string if not
     */
    createEvent: string;
    /**
     * String which contains event if we have permission to delete user of empty string if not
     */
    deleteEvent: string;
    /**
     * String which contains event if we have permission to update user of empty string if not
     */
    updateEvent: string;
    /**
     * entityID used to control the definition of an optional entity column which is created with the {@link Users.entityColumn} function.
     * @internal
     */
    entityID: string;
    /**
     * entityID used to display label from the selector of entityID {@link Users.entityColumn} function.
     * @internal
     */
    entityLabelKey: string;
    /**
     * entityID used to control the definition of an optional entity column which is created with the {@link Users.entityColumn} function.
     * @internal
     */
    entityLabel: string;
    /**
     * Definition of the `DATASERVER` query which is used when managing Counterpartys
     * @internal
     */
    availableEntitiesEndpoint: string;
    /**
     * Constructor will set the {@link Users.createEvent} {@link Users.deleteEvent} {@link Users.updateEvent} to match the permissions set from the server
     */
    constructor();
    connectedCallback(): Promise<void>;
    disconnectedCallback(): Promise<void>;
    deepClone(): Node;
    /**
     * Used as the callback function in {@link Users.statusColumn}
     * @internal
     */
    statusRendererSelector(params: any): {
        component: GridProRendererTypes;
        params: {
            actionClick: () => Promise<void>;
            actionName: string;
            appearance: string;
        };
    };
    /**
     * Changes the status of an entity
     *
     * @remarks
     *
     * The action which is run on the status button from {@link Users.statusRendererSelector}
     *
     * @internal
     */
    changeStatus(params: any): Promise<void>;
    /**
     * @internal
     */
    readUserData(user: any): Promise<{
        USER_PROFILES: any[];
        ADDRESS_LINE_1: any;
        ADDRESS_LINE_2: any;
        ADDRESS_LINE_3: any;
        ADDRESS_LINE_4: any;
        ACCESS_TYPE: string;
    } | {
        USER_PROFILES: any[];
        ADDRESS_LINE_1: any;
        ADDRESS_LINE_2: any;
        ADDRESS_LINE_3: any;
        ADDRESS_LINE_4: any;
        ACCESS_TYPE?: undefined;
    }>;
}

declare const Users_base: (new (...args: any[]) => {
    "__#1@#_container": FoundationLayoutContainer | LayoutCacheContainer | DOMContainer;
    "__#1@#_latestTokenCode": string;
    "__#1@#_hasFirstLoaded": boolean;
    "__#1@#_cleanupTimeout": NodeJS.Timeout;
    "__#1@#_shouldForceLifecycle": boolean;
    cloneNode(deep?: boolean): Node;
    deepClone(): Node;
    readonly shouldRunDisconnect: boolean;
    readonly shouldRunConnect: boolean;
    "__#1@#_blockLifecycleDueToTokenChange"(lifecycleType: "connect" | "disconnect" | "reconnect"): boolean;
    "__#1@#_tryFindContainingLayout"(e: Element): FoundationLayoutContainer | LayoutCacheContainer | DOMContainer;
    connectedCallback(): void;
    readonly $fastController: Controller;
    $emit(type: string, detail?: any, options?: Omit<CustomEventInit<any>, "detail">): boolean | void;
    disconnectedCallback(): void;
    attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
    accessKey: string;
    readonly accessKeyLabel: string;
    autocapitalize: string;
    dir: string;
    draggable: boolean;
    hidden: boolean;
    inert: boolean;
    innerText: string;
    lang: string;
    readonly offsetHeight: number;
    readonly offsetLeft: number;
    readonly offsetParent: Element;
    readonly offsetTop: number;
    readonly offsetWidth: number;
    outerText: string;
    /**
     * GridOptions to be passed down from application
     * @public
     */
    spellcheck: boolean;
    title: string;
    translate: boolean;
    attachInternals(): ElementInternals;
    click(): void;
    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
    removeEventListener<K_1 extends keyof HTMLElementEventMap>(type: K_1, listener: (this: HTMLElement, ev: HTMLElementEventMap[K_1]) => any, options?: boolean | EventListenerOptions): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
    readonly attributes: NamedNodeMap;
    readonly classList: DOMTokenList;
    className: string;
    readonly clientHeight: number;
    readonly clientLeft: number;
    readonly clientTop: number;
    readonly clientWidth: number;
    id: string;
    readonly localName: string;
    readonly namespaceURI: string;
    onfullscreenchange: (this: Element, ev: Event) => any;
    onfullscreenerror: (this: Element, ev: Event) => any;
    outerHTML: string;
    readonly ownerDocument: Document;
    readonly part: DOMTokenList;
    readonly prefix: string;
    readonly scrollHeight: number;
    scrollLeft: number;
    scrollTop: number;
    readonly scrollWidth: number;
    readonly shadowRoot: ShadowRoot;
    /**
     * String which contains event if we have permission to update user of empty string if not
     */
    slot: string;
    readonly tagName: string;
    attachShadow(init: ShadowRootInit): ShadowRoot;
    closest<K_2 extends keyof HTMLElementTagNameMap>(selector: K_2): HTMLElementTagNameMap[K_2];
    closest<K_3 extends keyof SVGElementTagNameMap>(selector: K_3): SVGElementTagNameMap[K_3];
    closest<E extends Element = Element>(selectors: string): E;
    getAttribute(qualifiedName: string): string;
    getAttributeNS(namespace: string, localName: string): string;
    getAttributeNames(): string[];
    getAttributeNode(qualifiedName: string): Attr;
    getAttributeNodeNS(namespace: string, localName: string): Attr;
    getBoundingClientRect(): DOMRect;
    getClientRects(): DOMRectList;
    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;
    getElementsByTagName<K_4 extends keyof HTMLElementTagNameMap>(qualifiedName: K_4): HTMLCollectionOf<HTMLElementTagNameMap[K_4]>;
    getElementsByTagName<K_5 extends keyof SVGElementTagNameMap>(qualifiedName: K_5): HTMLCollectionOf<SVGElementTagNameMap[K_5]>;
    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;
    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;
    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;
    getElementsByTagNameNS(namespace: string, localName: string): HTMLCollectionOf<Element>;
    hasAttribute(qualifiedName: string): boolean;
    hasAttributeNS(namespace: string, localName: string): boolean;
    hasAttributes(): boolean;
    hasPointerCapture(pointerId: number): boolean;
    insertAdjacentElement(where: InsertPosition, element: Element): Element;
    insertAdjacentHTML(position: InsertPosition, text: string): void;
    insertAdjacentText(where: InsertPosition, data: string): void;
    matches(selectors: string): boolean;
    releasePointerCapture(pointerId: number): void;
    removeAttribute(qualifiedName: string): void;
    removeAttributeNS(namespace: string, localName: string): void;
    removeAttributeNode(attr: Attr): Attr;
    requestFullscreen(options?: FullscreenOptions): Promise<void>;
    requestPointerLock(): void;
    scroll(options?: ScrollToOptions): void;
    scroll(x: number, y: number): void;
    scrollBy(options?: ScrollToOptions): void;
    scrollBy(x: number, y: number): void;
    scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;
    scrollTo(options?: ScrollToOptions): void;
    scrollTo(x: number, y: number): void;
    setAttribute(qualifiedName: string, value: string): void;
    setAttributeNS(namespace: string, qualifiedName: string, value: string): void;
    setAttributeNode(attr: Attr): Attr;
    setAttributeNodeNS(attr: Attr): Attr; /**
    * Used as the callback function in {@link Users.statusColumn}
    * @internal
    */
    setPointerCapture(pointerId: number): void;
    toggleAttribute(qualifiedName: string, force?: boolean): boolean;
    webkitMatchesSelector(selectors: string): boolean;
    readonly baseURI: string;
    readonly childNodes: NodeListOf<ChildNode>;
    readonly firstChild: ChildNode;
    readonly isConnected: boolean;
    readonly lastChild: ChildNode;
    readonly nextSibling: ChildNode;
    readonly nodeName: string;
    readonly nodeType: number;
    nodeValue: string;
    readonly parentElement: HTMLElement;
    readonly parentNode: ParentNode;
    readonly previousSibling: ChildNode;
    textContent: string;
    appendChild<T_1 extends Node>(node: T_1): T_1;
    compareDocumentPosition(other: Node): number;
    contains(other: Node): boolean;
    getRootNode(options?: GetRootNodeOptions): Node;
    hasChildNodes(): boolean;
    insertBefore<T_2 extends Node>(node: T_2, child: Node): T_2;
    isDefaultNamespace(namespace: string): boolean;
    isEqualNode(otherNode: Node): boolean;
    isSameNode(otherNode: Node): boolean;
    lookupNamespaceURI(prefix: string): string;
    lookupPrefix(namespace: string): string;
    normalize(): void;
    removeChild<T_3 extends Node>(child: T_3): T_3;
    replaceChild<T_4 extends Node>(node: Node, child: T_4): T_4;
    readonly ATTRIBUTE_NODE: number;
    readonly CDATA_SECTION_NODE: number;
    readonly COMMENT_NODE: number;
    readonly DOCUMENT_FRAGMENT_NODE: number;
    readonly DOCUMENT_NODE: number;
    readonly DOCUMENT_POSITION_CONTAINED_BY: number;
    readonly DOCUMENT_POSITION_CONTAINS: number;
    readonly DOCUMENT_POSITION_DISCONNECTED: number;
    readonly DOCUMENT_POSITION_FOLLOWING: number;
    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;
    readonly DOCUMENT_POSITION_PRECEDING: number;
    readonly DOCUMENT_TYPE_NODE: number;
    readonly ELEMENT_NODE: number;
    readonly ENTITY_NODE: number;
    readonly ENTITY_REFERENCE_NODE: number;
    readonly NOTATION_NODE: number;
    readonly PROCESSING_INSTRUCTION_NODE: number;
    readonly TEXT_NODE: number;
    dispatchEvent(event: Event): boolean;
    ariaAtomic: string;
    ariaAutoComplete: string;
    ariaBusy: string;
    ariaChecked: string;
    ariaColCount: string;
    ariaColIndex: string;
    ariaColIndexText: string;
    ariaColSpan: string;
    ariaCurrent: string;
    ariaDisabled: string;
    ariaExpanded: string;
    ariaHasPopup: string;
    ariaHidden: string;
    ariaInvalid: string;
    ariaKeyShortcuts: string;
    ariaLabel: string;
    ariaLevel: string;
    ariaLive: string;
    ariaModal: string;
    ariaMultiLine: string;
    ariaMultiSelectable: string;
    ariaOrientation: string;
    ariaPlaceholder: string;
    ariaPosInSet: string;
    ariaPressed: string;
    ariaReadOnly: string;
    ariaRequired: string;
    ariaRoleDescription: string;
    ariaRowCount: string;
    ariaRowIndex: string;
    ariaRowIndexText: string;
    ariaRowSpan: string;
    ariaSelected: string;
    ariaSetSize: string;
    ariaSort: string;
    ariaValueMax: string;
    ariaValueMin: string;
    ariaValueNow: string;
    ariaValueText: string;
    role: string;
    animate(keyframes: PropertyIndexedKeyframes | Keyframe[], options?: number | KeyframeAnimationOptions): Animation;
    getAnimations(options?: GetAnimationsOptions): Animation[];
    after(...nodes: (string | Node)[]): void;
    before(...nodes: (string | Node)[]): void;
    remove(): void;
    replaceWith(...nodes: (string | Node)[]): void;
    innerHTML: string;
    readonly nextElementSibling: Element;
    readonly previousElementSibling: Element;
    readonly childElementCount: number;
    readonly children: HTMLCollection;
    readonly firstElementChild: Element;
    readonly lastElementChild: Element;
    append(...nodes: (string | Node)[]): void;
    prepend(...nodes: (string | Node)[]): void;
    querySelector<K_6 extends keyof HTMLElementTagNameMap>(selectors: K_6): HTMLElementTagNameMap[K_6];
    querySelector<K_7 extends keyof SVGElementTagNameMap>(selectors: K_7): SVGElementTagNameMap[K_7];
    querySelector<E_1 extends Element = Element>(selectors: string): E_1;
    querySelectorAll<K_8 extends keyof HTMLElementTagNameMap>(selectors: K_8): NodeListOf<HTMLElementTagNameMap[K_8]>;
    querySelectorAll<K_9 extends keyof SVGElementTagNameMap>(selectors: K_9): NodeListOf<SVGElementTagNameMap[K_9]>;
    querySelectorAll<E_2 extends Element = Element>(selectors: string): NodeListOf<E_2>;
    replaceChildren(...nodes: (string | Node)[]): void;
    readonly assignedSlot: HTMLSlotElement;
    oncopy: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    oncut: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    onpaste: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
    readonly style: CSSStyleDeclaration;
    contentEditable: string;
    enterKeyHint: string;
    inputMode: string;
    readonly isContentEditable: boolean;
    onabort: (this: GlobalEventHandlers, ev: UIEvent) => any;
    onanimationcancel: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationend: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationiteration: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onanimationstart: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
    onauxclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onbeforeinput: (this: GlobalEventHandlers, ev: InputEvent) => any;
    onblur: (this: GlobalEventHandlers, ev: FocusEvent) => any;
    oncancel: (this: GlobalEventHandlers, ev: Event) => any;
    oncanplay: (this: GlobalEventHandlers, ev: Event) => any;
    oncanplaythrough: (this: GlobalEventHandlers, ev: Event) => any;
    onchange: (this: GlobalEventHandlers, ev: Event) => any;
    onclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onclose: (this: GlobalEventHandlers, ev: Event) => any;
    oncontextmenu: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    oncuechange: (this: GlobalEventHandlers, ev: Event) => any;
    ondblclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    ondrag: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragend: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragenter: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragleave: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragover: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondragstart: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondrop: (this: GlobalEventHandlers, ev: DragEvent) => any;
    ondurationchange: (this: GlobalEventHandlers, ev: Event) => any;
    onemptied: (this: GlobalEventHandlers, ev: Event) => any;
    onended: (this: GlobalEventHandlers, ev: Event) => any;
    onerror: OnErrorEventHandlerNonNull;
    onfocus: (this: GlobalEventHandlers, ev: FocusEvent) => any;
    onformdata: (this: GlobalEventHandlers, ev: FormDataEvent) => any;
    ongotpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    oninput: (this: GlobalEventHandlers, ev: Event) => any;
    oninvalid: (this: GlobalEventHandlers, ev: Event) => any;
    onkeydown: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onkeypress: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onkeyup: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
    onload: (this: GlobalEventHandlers, ev: Event) => any;
    onloadeddata: (this: GlobalEventHandlers, ev: Event) => any;
    onloadedmetadata: (this: GlobalEventHandlers, ev: Event) => any;
    onloadstart: (this: GlobalEventHandlers, ev: Event) => any;
    onlostpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onmousedown: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseenter: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseleave: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmousemove: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseout: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseover: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onmouseup: (this: GlobalEventHandlers, ev: MouseEvent) => any;
    onpause: (this: GlobalEventHandlers, ev: Event) => any;
    onplay: (this: GlobalEventHandlers, ev: Event) => any;
    onplaying: (this: GlobalEventHandlers, ev: Event) => any;
    onpointercancel: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerdown: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerenter: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerleave: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointermove: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerout: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerover: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onpointerup: (this: GlobalEventHandlers, ev: PointerEvent) => any;
    onprogress: (this: GlobalEventHandlers, ev: ProgressEvent<EventTarget>) => any;
    onratechange: (this: GlobalEventHandlers, ev: Event) => any;
    onreset: (this: GlobalEventHandlers, ev: Event) => any;
    onresize: (this: GlobalEventHandlers, ev: UIEvent) => any;
    onscroll: (this: GlobalEventHandlers, ev: Event) => any;
    onsecuritypolicyviolation: (this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any;
    onseeked: (this: GlobalEventHandlers, ev: Event) => any;
    onseeking: (this: GlobalEventHandlers, ev: Event) => any;
    onselect: (this: GlobalEventHandlers, ev: Event) => any;
    onselectionchange: (this: GlobalEventHandlers, ev: Event) => any;
    onselectstart: (this: GlobalEventHandlers, ev: Event) => any;
    onslotchange: (this: GlobalEventHandlers, ev: Event) => any;
    onstalled: (this: GlobalEventHandlers, ev: Event) => any;
    onsubmit: (this: GlobalEventHandlers, ev: SubmitEvent) => any;
    onsuspend: (this: GlobalEventHandlers, ev: Event) => any;
    ontimeupdate: (this: GlobalEventHandlers, ev: Event) => any;
    ontoggle: (this: GlobalEventHandlers, ev: Event) => any;
    ontouchcancel?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchend?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchmove?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontouchstart?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
    ontransitioncancel: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionend: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionrun: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    ontransitionstart: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
    onvolumechange: (this: GlobalEventHandlers, ev: Event) => any;
    onwaiting: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationend: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationiteration: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkitanimationstart: (this: GlobalEventHandlers, ev: Event) => any;
    onwebkittransitionend: (this: GlobalEventHandlers, ev: Event) => any;
    onwheel: (this: GlobalEventHandlers, ev: WheelEvent) => any;
    autofocus: boolean;
    readonly dataset: DOMStringMap;
    nonce?: string;
    tabIndex: number;
    blur(): void;
    focus(options?: FocusOptions): void;
}) & (new () => HTMLElement & FASTElement) & {
    from<TBase extends {
        new (): HTMLElement;
        prototype: HTMLElement;
    }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
    define<TType extends Function>(type: TType, nameOrDef?: string | PartialFASTElementDefinition): TType;
};

/**
 * Defines the default columns which are present on the user management component
 *
 * @remarks
 *
 * This is exported so you can override and mix-in custom columns along with the default
 * @public
 */
export declare const UsersColumnConfig: ({
    field: string;
    headerName: string;
    enableCellChangeFlash: boolean;
} | {
    field: string;
    headerName: string;
    valueFormatter: ({ value }: {
        value: any;
    }) => string;
    enableCellChangeFlash: boolean;
})[];

export { }
